// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>

#pragma pack(1)       // set byte packing
typedef struct {
	unsigned __int32 bLength;
	char szSerNum[9];
	char szProdName[9];
	unsigned __int64 MinFreq;
	unsigned __int64 MaxFreq;
	struct {
		unsigned __int32 ExtRef:1;
		unsigned __int32 Reserved:31;
	} Features;
} RADIO_INFO2;
#pragma pack()       // set back the default packing

// G3 API function type declarations
typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCGetInfo)(int hRadio,RADIO_INFO2 *info);
typedef BOOL (__stdcall *FNCSetAtten)(int hRadio, BOOL fAtten);
typedef BOOL (__stdcall *FNCSetPower)(int hRadio, BOOL fPower);
typedef BOOL (__stdcall *FNCSetAGC)(int hRadio, int iAGC);
typedef BOOL (__stdcall *FNCBlockScan)(int hRadio,DWORD *Freqs,size_t Count,int StopSquelchRaw,DWORD FeedbackTime,HWND WinHandle,DWORD Msg);
typedef BOOL (__stdcall *FNCStopBlockScan)(int hRadio);
typedef BOOL (__stdcall *FNCCodecStart)(int hRadio,void (__stdcall *CallbackFunc)(void *),void *CallbackTarget);
typedef BOOL (__stdcall *FNCCodecStop)(int hRadio);

int main(int argc, char* argv[])
{
	// load the G3 API library
	HMODULE dll=LoadLibrary("wrg303api.dll");
	if (!dll) {
		puts("WRG303API.DLL not found !"); 
		return 0;
	}

	// link G3 API functions
	FNCOpenRadioDevice OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(dll,"OpenRadioDevice");
	FNCCloseRadioDevice CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(dll,"CloseRadioDevice");
	FNCGetInfo GetInfo=(FNCGetInfo)GetProcAddress(dll,"GetInfo");
	FNCSetAtten SetAtten=(FNCSetAtten)GetProcAddress(dll,"SetAtten");
	FNCSetPower SetPower=(FNCSetPower)GetProcAddress(dll,"SetPower");
	FNCSetAGC SetAGC=(FNCSetAGC)GetProcAddress(dll,"SetAGC");
	FNCBlockScan BlockScan=(FNCBlockScan)GetProcAddress(dll,"BlockScan");
	FNCStopBlockScan StopBlockScan=(FNCStopBlockScan)GetProcAddress(dll,"StopBlockScan");
	FNCCodecStart CodecStart=(FNCCodecStart)GetProcAddress(dll,"CodecStart");
	FNCCodecStop CodecStop=(FNCCodecStop)GetProcAddress(dll,"CodecStop");

	if (!GetInfo || !BlockScan || !StopBlockScan) {
		puts("Not all needed API function not found. Please update the WR-G303 software!");
		FreeLibrary(dll);
		return 0;
	}

	// open the first available radio
	int hRadio=OpenRadioDevice(0);
	if (!hRadio) {
		puts("No WR-G303 device could be opened !");
		FreeLibrary(dll); 
		return 0;
	}

	// obtain the radio info to be able to detect USB connected receivers
	RADIO_INFO2 radio_info;
	radio_info.bLength=sizeof(radio_info);
	GetInfo(hRadio,&radio_info);

	// check for USB receiver
	BOOL USB_recv=FALSE;
	if (!memcmp(radio_info.szProdName,"G3USBv",6)) {
		puts("USB receiver detected");
		USB_recv=TRUE;
	}

	// set Power,AGC and Attenuator
	if (SetPower(hRadio,TRUE)) puts("The device is turned ON");
	else puts("The device failed to turn ON");
	if (SetAtten(hRadio,FALSE)) puts("The attenuator is OFF");
	else puts("The attenuator failed to turn OFF");
	if (SetAGC(hRadio,3)) puts("The AGC is FAST");
	else puts("The AGC failed to switch");

	// for USB receivers codec streaming must be started
	if (USB_recv) CodecStart(hRadio,NULL,NULL);

	// prepare the buffer for the blockscan from 600 to 800 kHz
	size_t i;
	DWORD Freqs[1001];  // buffer for frequencies to scan
	for (i=0;i<=1000;i++) Freqs[i]=(DWORD)(600000+200*i);

	// let's start the blockscan with 1001 frequencies, don't stop by Squelch.
	// no feedback (after a million ms), don't send us the buffers, we'll check that all
	if (BlockScan(hRadio,Freqs,1001,256,1000000,NULL,0)) puts("The block scanning started..");
	else puts("The block scanning failed to start");

	puts("Scanned data:");
	for (i=0;i<=1000;i++) {
		while (Freqs[i]>255) Sleep(10);    // the frequency is not scanned yet, we have to wait
		printf("Freq: %ld\tRAW:%ld\n",600000+i*200,Freqs[i]);
	}

	// the block scanning is over
	if (StopBlockScan(hRadio)) puts("Block scanning is stopped");
	else puts("Block scanning couldn't stop");

	// for USB receivers codec streaming must be stopped
	if (USB_recv) CodecStop(hRadio);

	// close the device handle
	if (CloseRadioDevice(hRadio)) puts("The device is closed properly");
	else puts("The device failed to close");

	// free the G3 API library
	FreeLibrary(dll);

	return 0;
}